-- Require packages
require "obj/cards/card"

-- The wildCard events
local _IF = {
	--- Increases entity health
	-- @param eID: 		entity ID
	-- @param power:	strength of effect
	potion =
		function(eID, power)
			entities[eID].ID.health = entities[eID].health + power
		end,
	--- Decreases opposing entity health
	-- @param eID:		opposing entity ID
	-- @param power:	stength of effect
	poison = 
		function(eID, power)
			entities[eID].ID.health = entities[eID].ID.health - power
		end,
	--- Increases a particular entity stat
	-- @param eID:		entity ID
	-- @param kind:		stat to change
	-- @param power:	strength of effect
	stimulant =
		function(eID, kind, power)
			entities[eID].ID[kind] = entities[eID].ID[kind] + power
		end,
	--- Decreases a particular opposing entity stat
	-- @param eID:		opposing entity ID
	-- @param kind:		stat to change
	-- @param power:	strength of effect
	trap =
		function(eID, kind, power)
			entities[eID].ID[kind] = entities[eID].ID[kind] - power
		end,
}

-- The random weights
local _RW = {
	potion = 0.25,
	poison = 0.25,
	stimulant = 0.25,
	trap = 0.25,
}

function newWildCard(x, y, w, h, eID, rWeightMods)
	local _IF = {
		--- Increases entity health
		-- @param eID: 		entity ID
		-- @param power:	strength of effect
		potion =
			function(eID, power)
				entities[eID].ID.health = entities[eID].health + power
			end,
		--- Decreases opposing entity health
		-- @param eID:		opposing entity ID
		-- @param power:	stength of effect
		poison = 
			function(eID, power)
				entities[eID].ID.health = entities[eID].ID.health - power
			end,
		--- Increases a particular entity stat
		-- @param eID:		entity ID
		-- @param kind:		stat to change
		-- @param power:	strength of effect
		stimulant =
			function(eID, kind, power)
				entities[eID].ID[kind] = entities[eID].ID[kind] + power
			end,
		--- Decreases a particular opposing entity stat
		-- @param eID:		opposing entity ID
		-- @param kind:		stat to change
		-- @param power:	strength of effect
		trap =
			function(eID, kind, power)
				entities[eID].ID[kind] = entities[eID].ID[kind] - power
			end,
	}

	local function getKind(rWMods)
		local step1 = (_RW.potion + rWMods.potion)
		local step2 = step1 + (_RW.poison + rWMods.poison)
		local step3 = step2 + (_RW.stimulant +rWMods.stimulant)
		local rand = math.random()
		if rand < step1 then
			return "potion"
		elseif step1 <= rand and rand < step2 then
			return "poison"
		elseif step2 <= rand and rand < step3 then
			return "stimulant"
		elseif step3 <= rand and rand < 1 then
			return "trap"
		end
	end
	
	local kind = getKind(rWeightMods)
	local power = math.random(3)
	local image = wildCardImages[kind][power]
	local self = newCard(x, y, w, h, image, eID)
	self.kind = kind
	self.event = _IF[kind]
	self.power = power
	
	local function sendEvent()
		table.insert(events, self.event)
	end
	
	local interface = {
		x = self.x,
		y = self.y,
		w = self.w,
		h = self.h,
		eID = self.eID,
		draw = self.draw,
		sendEvent = sendEvent,
	}
	return interface
end
--~ 
--~ do
	--~ -- The wildCard metatable
	--~ local wildCard = {}
	--~ wildCard.__tostring = "wildcard"
	--~ wildCard.__index = wildCard		-- Gives inherentance to card
--~ 
	--~ -- The wildCard events
	--~ local _IF = {
		--~ --- Increases entity health
		--~ -- @param eID: 		entity ID
		--~ -- @param power:	strength of effect
		--~ potion =
			--~ function(eID, power)
				--~ entities[eID].ID.health = entities[eID].health + power
			--~ end,
		--~ --- Decreases opposing entity health
		--~ -- @param eID:		opposing entity ID
		--~ -- @param power:	stength of effect
		--~ poison = 
			--~ function(eID, power)
				--~ entities[eID].ID.health = entities[eID].ID.health - power
			--~ end,
		--~ --- Increases a particular entity stat
		--~ -- @param eID:		entity ID
		--~ -- @param kind:		stat to change
		--~ -- @param power:	strength of effect
		--~ stimulant =
			--~ function(eID, kind, power)
				--~ entities[eID].ID[kind] = entities[eID].ID[kind] + power
			--~ end,
		--~ --- Decreases a particular opposing entity stat
		--~ -- @param eID:		opposing entity ID
		--~ -- @param kind:		stat to change
		--~ -- @param power:	strength of effect
		--~ trap =
			--~ function(eID, kind, power)
				--~ entities[eID].ID[kind] = entities[eID].ID[kind] - power
			--~ end,
	--~ }
	--~ 
	--~ -- The random weights
	--~ local _RW = {
		--~ potion = 0.25,
		--~ poison = 0.25,
		--~ stimulant = 0.25,
		--~ trap = 0.25,
	--~ }
	--~ 
	--~ --- Gets the type of wildCard
	--~ -- @param rWMods: the weight modifications for the type percentages
	--~ local function getKind(rWMods)
		--~ local step1 = (_RW.potion + rWMods.potion)
		--~ local step2 = step1 + (_RW.poison + rWMods.poison)
		--~ local step3 = step2 + (_RW.stimulant +rWMods.stimulant)
		--~ local rand = math.random()
		--~ if rand < step1 then
			--~ return "potion"
		--~ elseif step1 <= rand and rand < step2 then
			--~ return "poison"
		--~ elseif step2 <= rand and rand < step3 then
			--~ return "stimulant"
		--~ elseif step3 <= rand and rand < 1 then
			--~ return "trap"
		--~ end
	--~ end
	--~ 
	--~ --- Creates a new wildCard
	--~ -- @param x:			x-coordinate
	--~ -- @param y:			y-coordinate
	--~ -- @param w:			width
	--~ -- @param h:			height
	--~ -- @param eID:			entity this card is associated with
	--~ -- @param rWeightMods:	entity's random weight modifications
	--~ function wildCard:new(x, y, w, h, eID, rWeightMods)
		--~ local kind = getKind(rWeightMods)
		--~ local power = math.random(3)
		--~ local image = wildCardImages[kind][power]
		--~ local w = cardLib:new(x, y, w, h, image)
		--~ w.kind = kind
		--~ w.event = _IF[kind]
		--~ w.power = power
		--~ return w
	--~ end
	--~ 
	--~ --- Sends the wildCard event to the event table to be eaten
	--~ function wildCard:sendEvent()
		--~ table.insert(events, self.event)
	--~ end
	--~ 
	--~ function wildCard:draw()
		--~ cardLib.card:draw()
	--~ end
	--~ 
	--~ -- The wildcard.lua namespace
	--~ local namespace = {
		--~ wildCard = wildCard
	--~ }
	--~ return namespace
	--~ return wildCard
--~ end
