local entityLib = require "obj/entity/entity"

do
	-- The placard metatable
	placard = {}
	placard.__index = placard
	
	local _PLACPOS = {
		_IMG = {x = 5, y = 5, w = 105, h = 210},
		_ATTR = {
			_HP = {x = 115, y = 25, w = 105, h = 50},
			_ATT = {x = 115, y = 80, w = 105, h = 50},
			_ATTPRINT = {x = 165, y = 80},
			_DEF = {x = 115, y = 135, w = 105, h = 50},
			_DEFPRINT = {x = 165, y = 135},
		},
	}
	
	local smallAttIcon = love.graphics.newImage("resources/images/staticons/smallatticon.png")
	local smallDefIcon = love.graphics.newImage("resources/images/staticons/smalldeficon.png")
	local smallHPIcon = love.graphics.newImage("resources/images/staticons/smallhpicon.png")
	
	--- Creates a new placard
	-- @param entity:		the entity this placard represents
	-- @param entityIcon:	the visual icon of the entity
	-- @param equipment:	the visual icon of the equipment the entity is wearing
	function placard:new(entity, entityIcon, equipIcon)
		local p = {
			hp = entity.hp,
			_MAXHP = entity.hp,
			att = entity.att,
			def = entity.def,
			entityIcon = entityIcon,
		}
		if equipIcon ~= nil then
			p.equipIcon = equipIcon
		end
		return setmetatable(p, placard)
	end
	
	function placard:draw(x, y)
		--~ love.graphics.setRenderTarget(fb)
		love.graphics.draw(
			self.entityIcon,
			x + _PLACPOS._IMG.x,
			y + _PLACPOS._IMG.y)
		love.graphics.draw(
			smallHPIcon,
			x + _PLACPOS._ATTR._HP.x,
			y + _PLACPOS._ATTR._HP.y)
		love.graphics.draw(
			smallAttIcon,
			x + _PLACPOS._ATTR._ATT.x,
			y + _PLACPOS._ATTR._ATT.y)
		love.graphics.draw(
			smallDefIcon,
			x + _PLACPOS._ATTR._DEF.x,
			y + _PLACPOS._ATTR._DEF.y)
		love.graphics.setColor(0,0,0,255)
		love.graphics.print(
			self.att,
			x + _PLACPOS._ATTR._ATTPRINT.x,
			y +_PLACPOS._ATTR._ATTPRINT.y)
		love.graphics.print(
			self.def,
			x +_PLACPOS._ATTR._DEFPRINT.x,
			y +_PLACPOS._ATTR._DEFPRINT.y)
		love.graphics.setColor(255,255,255,255)
		--~ love.graphics.setRenderTarget()
		--~ return fb
	end
	
	-- The placard.lua namespace
	local namespace = {
		placard = placard
	}
	return namespace
end
