do
	-- The item metatable
	local item = {}
	item.__index = item
	
	-- The naming standards for the varios items
	_STAND = {
		-- Attack-based battlesets
		_ATT = {
			"Fine ",	-- Attack +1, Defense +0
			"Sharp ",	-- Attack +2, Defense -1
			"Deadly ",	-- Attack +3, Defense -2
		},
		-- Defense-based battlesets
		_DEF = {
			"Leather ",	-- Defense +1, Attack +0
			"Iron ",	-- Defense +2, Attack -1
			"Steel ",	-- Defense +3, Attack -2
		},
		-- The trinket strengths
		_TRINK = {
			"Small ",	-- Random +5% one stat
			"Large ",	-- Random +7.5% one stat
			"Robust ",	-- Random +10% one stat
		},
	}
	
	--- Determines the type of item by random chance
	local function getKind()
		if math.random() < 0.5 then
			return "Battle Set"
		else
			return "Trinket"
		end
	end
	
	--- Gets the stats of a battleset, and gives it the correct name
	local function getBattleStat()
		-- Determine the attack mod
		local attMod = math.random(-2, 3)
		-- Based on the attack mod, get the defense mod so that
		-- attMod + defMod = 1
		local defMod = 1 - attMod
		local name = ""
		-- Pick the correct standards name based on whichever mod is
		-- dominant
		if attMod > defMod then
			name = _STAND._ATT[attMod]
			return name, attMod, defMod
		else
			name = _STAND._DEF[defMod]
			return name, attMod, defMod
		end
	end
	
	--- Gets the weights for the trinket, and gives it the correct name
	local function getTrinkStat()
		local rand = math.random()
		local pre = ""
		local randModAdd = 0
		-- The strength of the trinket is determined by chance:
		-- 		Weak (1): 	62.5%
		--		Medium (2):	32.25%
		--		Strong (3):	6.25%
		if rand < 10 / 16 then
			pre = _STAND._TRINK[1]
			randModAdd = 0.5
		elseif 10 / 16 <= rand and rand < 15 / 16 then
			pre = _STAND._TRINK[2]
			randModAdd = 0.75
		else
			pre = _STAND._TRINK[3]
			randModAdd = 1
		end
		-- Because percentage is 1/100, if I were to simply increase
		-- a perctange by adding to the range of possible values that
		-- would give me a particular result, the upper bound would
		-- break out of 100; therefore, I must subtract the increase
		-- from the remaining ranges evenly
		local randModSub = randModAdd / 3
		local name = ""
		local mod = {}
		rand = math.random(4)
		-- The type of trinket is a simple 25% chance.
		if rand == 1 then
			name = "Aid Kit"
			mod.potion, mod.poison, mod.stimulant, mod.trap =
				randModAdd, randModSub, randModSub, randModSub
		elseif rand == 2 then
			name = "Drug Pack"
			mod.potion, mod.poison, mod.stimulant, mod.trap =
				randModSub, randModAdd, randModSub, randModSub
		elseif rand == 3 then
			name = "Herb Pouch"
			mod.potion, mod.poison, mod.stimulant, mod.trap =
				randModSub, randModSub, randModAdd, randModSub
		elseif rand == 4 then
			name = "Trap Kit"
			mod.potion, mod.poison, mod.stimulant, mod.trap =
				randModSub, randModSub, randModSub, randModAdd
		end
		return pre .. name, mod
	end
	
	--- Creates a new item
	--- The item is determined completely by chance, and is created
	--- by chance upon a victory.
	function item:new()
		local kind = getKind()
		local i = {}
		if kind == "battleset" then
			local prefix = ""
			prefix, i.att, i.def = getBattleStat(kind)
			i.name = prefix .. kind
			i.statConstMod = {
				att = i.att * 0.1,
				def = i.def * 0.1,
			}
		elseif kind == "trinket" then
			i.name, trinkConstMod = getTrinkStat()
		end
		return setmetatable(i, item)
	end
	
	-- The item.lua namespace
	local namespace = {
		item = item
	}
	return namespace
end
