do
	-- Required packages
	require "obj/cards/card"
	require "obj/cards/statcard"
	require "obj/cards/wildcard"
	require "obj/entity/entity"
	require "obj/entity/player"
	require "obj/entity/npc"
	require "obj/entity/placard/placard"
	require "obj/inventory/item"
	require "states/battle/stage"
	
	-- The positions of the cards on the play space
	local _HANDPOS = {
		_P = {
			x = 25, y = 350, w = 750, h = 250,
			_ATT = {x = 283, y = 375},
			_DEF = {x = 451, y = 375},
			_WILD = {x = 619, y = 375},
			_PLACARD = {x = 40, y = 350}
		},
		_E = {
			x = 25, y = 0, w = 750, h = 250,
			_ATT = {x = 382, y = 25},
			_DEF = {x = 214, y = 25},
			_WILD = {x = 46, y = 25},
			_PLACARD = {x = 550, y = 0},
		},
	}
	local _CARDWIDTH = 150
	local _CARDHEIGHT = 200
	local selectionPhase = false
	local selectedKey = nil
	local timer = 0
	local damage = 0
	local placardFb = love.graphics.newFramebuffer()
	local cardHide = love.graphics.newImage("resources/images/cardhide.png")
	local stage = newStage()
	
	-- Temporary weights for wild card draws until I implement equipment
	-- system.
	local tempMods = {
		potion = 0,
		poison = 0,
		stimulant = 0,
		trap = 0,
	}
	
	-- Temporary player until I implement experience system.
	playerA = newPlayer(10, 5, 8)

	local function load()
		hideCards = true
		if statCardImages == nil or wildCardImages == nil then
			local temp = require "states/battle/loadcards"
			statCardImages, wildCardImages = temp[1], temp[2]
		end
		local playerImage = love.graphics.newImage("resources/images/player/minsc.png")
		local enemyImage = love.graphics.newImage("resources/images/player/halforc.png")
		placeholderScene = love.graphics.newImage("resources/images/samplescene.png")
		entities = {
			player = {
				ID = playerA,
				hand = {
					att = {},
					def = {},
					wild = {},
				},
			},
			enemy = {
				ID = newNPC(10, 7, 3, "Goblin", 25, .05),
				hand = {
					att = {},
					def = {},
					wild = {},
				},
			},
		}
		-- Renders the placards
		love.graphics.setRenderTarget(placardFb)
		love.graphics.setColor(255,0,0,255)
		love.graphics.rectangle("fill", _HANDPOS._P.x, _HANDPOS._P.y, _HANDPOS._P.w, _HANDPOS._P.h)
		love.graphics.rectangle("fill", _HANDPOS._E.x, _HANDPOS._E.y, _HANDPOS._E.w, _HANDPOS._E.h)
		love.graphics.setColor(255,255,255,255)
		entities.player.placard = placard:new(entities.player.ID, playerImage)
		entities.player.placard:draw(_HANDPOS._P._PLACARD.x, _HANDPOS._P._PLACARD.y)
		entities.enemy.placard = placard:new(entities.enemy.ID, enemyImage)
		entities.enemy.placard:draw(_HANDPOS._E._PLACARD.x, _HANDPOS._E._PLACARD.y)
		love.graphics.setRenderTarget()
		-- Simple events table
		events = {}
	end
	
	--- Draws a card from the deck (not drawing to the screen)
	-- @param kind:	the type of card
	-- @param eID:	the entity associated with the card
	local function drawCard(kind, eID)
		if eID == "player" then
			if kind == "att" then
				return newStatCard(
					_HANDPOS._P._ATT.x,
					_HANDPOS._P._ATT.y,
					_CARDWIDTH,
					_CARDHEIGHT,
					entities[eID].ID,
					kind)
			elseif kind == "def" then
				return newStatCard(
					_HANDPOS._P._DEF.x,
					_HANDPOS._P._DEF.y,
					_CARDWIDTH,
					_CARDHEIGHT,
					entities[eID].ID,
					kind)
			elseif kind == "wild" then
				return newWildCard(
					_HANDPOS._P._WILD.x,
					_HANDPOS._P._WILD.y,
					_CARDWIDTH,
					_CARDHEIGHT,
					entities[eID].ID,
					tempMods)
			end
		elseif eID == "enemy" then
			if kind == "att" then
				return newStatCard(
					_HANDPOS._E._ATT.x,
					_HANDPOS._E._ATT.y,
					_CARDWIDTH,
					_CARDHEIGHT,
					entities[eID].ID,
					kind)
			elseif kind == "def" then
				return newStatCard(
					_HANDPOS._E._DEF.x,
					_HANDPOS._E._DEF.y,
					_CARDWIDTH,
					_CARDHEIGHT,
					entities[eID].ID,
					kind)
			elseif kind == "wild" then
				return newWildCard(
					_HANDPOS._E._WILD.x,
					_HANDPOS._E._WILD.y,
					_CARDWIDTH,
					_CARDHEIGHT,
					entities[eID].ID,
					tempMods)
			end
		end
	end
	
	--- Adds a card to the stage
	-- @param k:	the type of card
	-- @param v:	the variables of the card
	-- @param eID:	the entity associated with the card
	function playCard(k, v, eID)
		print("play card")
		stage.add({kind = k, power = v.power}, eID)
		print(eID, k)
	end
	
	--- Replaces the empty cards in a hand with the appropriate
	--- card type
	local function deal()
		print("start deal")
		for k,v in pairs(entities.player.hand) do
			if next(v) == nil then
				print("draw player card", k)
				entities.player.hand[k] = drawCard(k, "player")
			end
		end
		for k,v in pairs(entities.enemy.hand) do
			if next(v) == nil then
				print("draw enemy card", k)
				entities.enemy.hand[k] = drawCard(k, "enemy")
			end
		end
		selectionPhase = true
		print("end deal")
	end
	
	--- One dimensional relative location detection
	--- Returns a negative if n < min, a positive if n > max, and 
	--- zero if min <= n <= max
	-- @param p:	number
	-- @param min:	lower bound
	-- @param max:	upper bound
	local function within(n, min, max)
		if n < min then return -1
		elseif n > max then return 1
		else return 0 end
	end
	
	--- Two dimensional location detection
	-- @param pos:	point in a coordinate plane
	-- @param r:	rectangle
	local function contains2D(pos, r)
		local xVal = within(pos.x, r.x, r.x + r.w)

		if xVal ~= 0 then return false end
		local yVal = within(pos.y, r.y, r.y + r.h)
		if yVal ~= 0 then return false end
		return true
	end
	
	--- Applies damage to an entity, based on the polarity of the value
	-- @param d: damage (positive or negative)
	local function applyDamage(d)
		if d < 0 then
			print("player hit")
			entities.player.ID.hp =
				entities.player.ID.hp - math.abs(d)
		elseif d > 0 then
			print("enemy hit")
			entities.enemy.ID.hp =
				entities.enemy.ID.hp - math.abs(d)
		end
	end
	
	--- Updates the battleState. Handles event calls.
	-- @param dt: delta time
	local function update(dt)
		for k,v in pairs(events) do
			if v == stage.run then
				print("Get damage")
				applyDamage(v())
				damage = 0
				deal()
			else
				v()
			end
			table.remove(events, k)
			print("event eaten")
		end
		--~ if damage ~= 0 then 
			--~ applyDamage(damage)
			--~ damage = 0
			--~ deal()
		--~ end
	end
	
	--- Draws the battleStage
	local function draw()
		love.graphics.setRenderTarget()
		love.graphics.draw(placardFb, 0, 0)
		love.graphics.draw(placeholderScene, 25, 250)
		for k,v in pairs(entities.player.hand) do
			v.draw()
		end
		for k,v in pairs(entities.enemy.hand) do
			v.draw()
		end
		if hideCards then
			love.graphics.draw(cardHide, _HANDPOS._P._WILD.x, _HANDPOS._P._WILD.y)
			love.graphics.draw(cardHide, _HANDPOS._E._WILD.x, _HANDPOS._E._WILD.y)
			love.graphics.draw(cardHide, _HANDPOS._E._ATT.x, _HANDPOS._E._ATT.y)
			love.graphics.draw(cardHide, _HANDPOS._E._DEF.x, _HANDPOS._E._DEF.y)
		end
	end
	
	--- Simple mouse control that sends a double-clicked card to the stage
	-- @callback x:	the mouse position's x-coordinate
	-- @callback y:	the mouse position's y-coordinate
	-- @callback b:	the button pressed
	function love.mousepressed(x, y, b)
		if b == "l" and selectionPhase then
			if selectedKey ~= nil and entities.player.hand[selectedKey].isSelected then
				playCard(selectedKey, entities.player.hand[selectedKey], "player")
				selectionPhase = false
				entities.enemy.ID.readHand(
					entities.player.ID,
					entities.enemy.hand)
				--~ table.insert(events, playCard(selectedKey, entities.player.hand[selectedKey], "player"))
			else
				for k,v in pairs(entities.player.hand) do
					if k ~= "wild" then
						if contains2D({x = x, y = y}, v) and not v.isSelected then
							v.isSelected = true
							selectedKey = k
						end
					end
				end
			end
		end
	end
	
	-- The battle.lua namespace
	local namespace = {
		load = load,
		deal = deal,
		update = update,
		draw = draw,
		mousepressed = mousepressed,
	}
	return namespace
end
