do
	-- Useful position information
	local _POS = {
		_CARD = {x = 0, y = 0, w = 150, h = 200},
		_ICON = {x = 60, y = 60, w = 90, h = 120},
		_POWER = {
			{x = 5, y = 145, w = 50, h = 50},
			{x = 95, y = 5, w = 50, h = 50},
		},
	}

	-- The artistic resources
	local cardBack = love.graphics.newImage("resources/images/cardback.png")
	local attIcon = love.graphics.newImage("resources/images/staticons/largeatticon.png")
	local defIcon = love.graphics.newImage("resources/images/staticons/largedeficon.png")
	local potionIcon = love.graphics.newImage("resources/images/wildicons/largepotion.png")
	local posionIcon = love.graphics.newImage("resources/images/wildicons/largepoison.png")
	local stimIcon = love.graphics.newImage("resources/images/wildicons/largestim.png")
	local trapIcon = love.graphics.newImage("resources/images/wildicons/largetrap.png")
	
	-- The stat card images
	local statCardImages = {
		att = {},
		def = {},
	}
	
	-- The wild card images
	local wildCardImages = {
		potion = {},
		poison = {},
		stimulant = {},
		trap = {},
	}
	
	--- Renders a single card onto the framebuffer
	-- @param fb:	framebuffer
	-- @param t:	the card type
	-- @param k:	the card subtype
	-- @param p:	the card power
	local function RenderCard(fb, t, k, p)
		love.graphics.setRenderTarget(fb)
		love.graphics.draw(cardBack, _POS._CARD.x, _POS._CARD.y)
		love.graphics.setColor(0,0,0,255)
		love.graphics.print(p, _POS._POWER[1].x, _POS._POWER[1].y)
		love.graphics.print(p, _POS._POWER[2].x, _POS._POWER[2].y)
		love.graphics.setColor(255,255,255,255)
		if t == "stat" then
			if k == "att" then
				love.graphics.draw(attIcon, _POS._ICON.x, _POS._ICON.y)
			elseif k == "def" then
				love.graphics.draw(defIcon, _POS._ICON.x, _POS._ICON.y)
			end
		elseif t == "wild" then
			if k == "potion" then
				love.graphics.draw(potionIcon, _POS._ICON.x, _POS._ICON.y)
			elseif k == "poison" then
				love.graphics.draw(posionIcon, _POS._ICON.x, _POS._ICON.y)
			elseif k == "stimulant" then
				love.graphics.draw(stimIcon, _POS._ICON.x, _POS._ICON.y)
			elseif k == "trap" then
				love.graphics.draw(trapIcon, _POS._ICON.x, _POS._ICON.y)
			end
		end
		love.graphics.setRenderTarget()
		return fb
	end
	
	-- Builds all the permutations of stat cards
	for k,_ in pairs(statCardImages) do
		for i = 1, 10 do
			local fb = love.graphics.newFramebuffer(_POS._CARD.w, _POS._CARD.h)
			fb = RenderCard(fb, "stat", k, i)
			statCardImages[k][i] = fb
		end
	end
	
	-- Builds all the permuations of wild cards
	for k,_ in pairs(wildCardImages) do
		for i = 1, 3 do
			local fb = love.graphics.newFramebuffer(_POS._CARD.w, _POS._CARD.h)
			fb = RenderCard(fb, "wild", k, i)
			wildCardImages[k][i] = fb
		end
	end

	-- The built image tables for each card type
	return {statCardImages, wildCardImages}
end
